/*
 * Created on Nov 13, 2003
 */
package MVC;

import java.awt.Toolkit;
import java.awt.datatransfer.DataFlavor;
import java.awt.datatransfer.StringSelection;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.io.File;

import javax.swing.JFileChooser;
import javax.swing.JOptionPane;
import javax.swing.text.BadLocationException;

import wikiviz.ArtifactList;

/**
 * @author wikiviz
 * Class controller implements Listeners for keyboard and mouse
 * input actions. The Observer is also implemented, but not used
 * so far.
 */
public class Controller implements KeyListener, ActionListener, Observer, MouseListener {

	/* (non-Javadoc)
		 * @see java.awt.event.KeyListener#keyPressed(java.awt.event.KeyEvent)
		 */

	public ApplicationMainView view; // The main window
	private ArtifactList model; // Model associated with the window
	private int numSimpleChars;
	private boolean requiresUpdateUndo;
	public int selectStart;
	public int selectEnd;
	private boolean requiresResetSelect;
	// and the controller
	JFileChooser fc; // File chooser dialog box

	// Default constructor for Controller
	public Controller() {
		numSimpleChars = 0;
		requiresResetSelect = true;
		selectStart =0;
		selectEnd = 0;
	}

	// Event generated when a key is pressed.
	public void keyPressed(KeyEvent evt) {
		view.textPane.setEditable(true);
		requiresUpdateUndo = true;
		String bulletString = new String("");
		int key = evt.getKeyCode(); // Keyboard code for the pressed key.
		int modifiers = evt.getModifiers(); // CTRL, SHIFT, ALT etc.
		requiresResetSelect = true;
		if ((modifiers & (KeyEvent.CTRL_DOWN_MASK | KeyEvent.CTRL_MASK))
			!= 0) {
			if (key == KeyEvent.VK_B) { // CTRL-B (Bold)
				model.handleBold();
			} else if (key == KeyEvent.VK_U) { // CTRL-U (Underline)
				model.handleUnderline();
			} else if (key == KeyEvent.VK_Z) { // CTRL-Z (Undo)
				if(view.isUndoEnabled())
					undo();
				requiresUpdateUndo = false;
				numSimpleChars--;
			} else if (key == KeyEvent.VK_I) { // CTRL-I (Italics)
				model.handleItalic();
			} else if (key == KeyEvent.VK_S) { // CTRL-S (Save)
				try {
					runSaveDialog();
				} catch (Exception ex) {
					// Bla - do nothing so far
				}
			} else if (key == KeyEvent.VK_L) { // CTRL-L Bullet start
				bulletString = handleBullet();
			} else if (key == KeyEvent.VK_M) { // CTRL-M Bullet end
				bulletString = handleEndBullet();
			}
			else if (key == KeyEvent.VK_V) {
				//Toolkit tk = new Toolkit();
				String clip = new String();
				try {
					clip = (String)(Toolkit.getDefaultToolkit().getSystemClipboard().getContents(new Object()).getTransferData(DataFlavor.stringFlavor));
				} catch (Exception e1) {
					
					e1.printStackTrace();
				} 
				for (int i = 0; i < clip.length(); i++) {
					model.appendChar(clip.charAt(i));
				}
				//tk.getSystemClipboard().getContents().getTransferData(DataFlavor.stringFlavor);
			}
			else if (key == KeyEvent.VK_X) {
				String cutText = model.cutSelection();
				StringSelection s = new StringSelection(cutText);
				Toolkit.getDefaultToolkit().getSystemClipboard().setContents(s,s);
			}
			// No more modifier from here on.
		} else if (
			(modifiers & (KeyEvent.SHIFT_DOWN_MASK | KeyEvent.SHIFT_MASK))
				!= 0) {
			if (key == KeyEvent.VK_TAB) { //SHIFT+TAB
				if (model.isBullet() && model.isCurrentArtifactEmpty()) {

					bulletString = handleEndBullet(); // End Bullet

				} else {
					requiresUpdateUndo = false;
					model.appendChar(evt.getKeyChar());
				}
			} else {
				char cha = (char) (evt.getKeyCode());
				if (cha != '') {
					if(cha == ';')
						cha = ':';
					model.appendChar(cha);
					requiresUpdateUndo = false;
					
					String temp = model.getCurrentArtifact().getStyleString();
					view.textPane.setEditable(false);
								try {
									int position = view.textPane.getCaretPosition();
													view.getStyledoc().insertString(position, String.valueOf(evt.getKeyChar()),
																					(view.getStyledoc()).getStyle(temp));
								} catch (BadLocationException e) {
									
									e.printStackTrace();
								}
			
				}
			}
		}else if (key == KeyEvent.VK_ENTER) { // Enter key
			bulletString = model.handleEnter();
			if (bulletString != "")
				evt.consume();
		} else if (key == KeyEvent.VK_BACK_SPACE) {
			model.handleBackspace(); // Backspace
		} else if (key == KeyEvent.VK_TAB) {
			if (model.isBullet() && model.isCurrentArtifactEmpty()) {
				bulletString = handleBullet();
				
			} else {
				model.appendChar(evt.getKeyChar());
				requiresUpdateUndo = false;
			}
			evt.consume();
			view.textPane.requestFocus();
		}else if (key == KeyEvent.VK_SPACE) {
			model.handleSpace();
		} else if (
			// Handle special keys
			key != KeyEvent.VK_SHIFT
				&& key != KeyEvent.VK_CONTROL
				&& key != KeyEvent.VK_ALT
				&& key != KeyEvent.VK_UP
				&& key != KeyEvent.VK_DOWN
				&& key != KeyEvent.VK_RIGHT
				&& key != KeyEvent.VK_LEFT) {
			model.appendChar(evt.getKeyChar());
			String temp = model.getCurrentArtifact().getStyleString();
			view.textPane.setEditable(false);
			try {
				int position = view.textPane.getCaretPosition();
				view.getStyledoc().insertString(position, String.valueOf(evt.getKeyChar()),
												(view.getStyledoc()).getStyle(temp));
			} catch (BadLocationException e) {
				
				e.printStackTrace();
			}			
		}
		try {
			view.getStyledoc().insertString(
				view.getStyledoc().getLength(),
				bulletString,
				view.getStyledoc().getStyle(
					"bold"));
		} catch (BadLocationException e) {
			
			e.printStackTrace();
		}
		if (requiresUpdateUndo){
			numSimpleChars = 0;
			model.updateUndoFile();
		}
		else
			numSimpleChars++;

	}
	/* 
	 * @see java.awt.event.KeyListener#keyTyped(java.awt.event.KeyEvent)
	 * No need to implement
	 */
	public void keyTyped(KeyEvent evt) {
	
	}

	/* 
	 * @see java.awt.event.KeyListener#keyReleased(java.awt.event.KeyEvent)
	 * No need to implement
	 */
	public void keyReleased(KeyEvent evt) {
		int key = evt.getKeyCode();
		
		 if (key == KeyEvent.VK_RIGHT || key == KeyEvent.VK_LEFT 
		|| key == KeyEvent.VK_UP || key == KeyEvent.VK_DOWN) { // Right arrow
				int position = view.textPane.getCaretPosition();
				model.updatePosition(position);
					
			} 
		selectStart = 0;
			selectEnd = 0;

	}

	/* 
	 * @see keyboardInput.Observer#update()
	 * Just needed to implement OBserver
	 * Does not do anything so far
	 */
	public void update() {
	}

	/**
	 * @return Model object
	 */
	public ArtifactList getModel() {
		return model;
	}

	/**
	 * @return View object (main window)
	 */
	public ApplicationMainView getView() {
		return view;
	}

	/**
	 * @param list (Set model)
	 */
	public void setModel(ArtifactList list) {
		model = list;
	}

	/**
	 * @param view (set view)
	 */
	public void setView(ApplicationMainView view) {
		this.view = view;
	}

	private String handleBullet() {
		String bulletString;
		bulletString = model.handleBullet(); // Bullet
		if (!bulletString.equals("") ) {
			try {
									view.getStyledoc().remove(
										view.getStyledoc().getLength()+2
											- model.getNumCurrentBulletChars(),
										model.getNumCurrentBulletChars()-2);
								} catch (BadLocationException e) {
									
									e.printStackTrace();
								}
		}
		return bulletString;
	}

	private String handleEndBullet() {
		String bulletString;
		bulletString = model.handleEndBullet();
		try {
			if(model.getNumCurrentBulletChars()==0) {
				view.getStyledoc().remove(view.getStyledoc().getLength()-1,1);
			}
			else
				view.getStyledoc().remove(
					view.getStyledoc().getLength()
						- model.getNumCurrentBulletChars(),
					model.getNumCurrentBulletChars());
		} catch (BadLocationException e) {
				
				e.printStackTrace();
		}
		return bulletString;
	}
	
	private void runHeaderDialog() {
		Object[] options = {"1","2","3","4", "5", "6"};
		int n = JOptionPane.showOptionDialog(view,
				"What depth header would you like? ",
				"Header Depth",
		JOptionPane.YES_NO_CANCEL_OPTION,
		JOptionPane.QUESTION_MESSAGE, null,options,	options[0]);
		model.handleHeader(n+1);
	}

	private void runSaveDialog() {
		fc = new JFileChooser();
		fc.setFileSelectionMode(JFileChooser.FILES_AND_DIRECTORIES);
		int returnVal = fc.showSaveDialog(view);
		if (returnVal == JFileChooser.APPROVE_OPTION) {
			File file = fc.getSelectedFile();
			try {
				model.outputWikiDoc(file.getPath());
			} catch (Exception ex) {
			}
		}
	}
	

	private void runLoadDialog() {
		fc = new JFileChooser();
		fc.setFileSelectionMode(JFileChooser.FILES_AND_DIRECTORIES);
		int returnVal = fc.showOpenDialog(view);
		if (returnVal == JFileChooser.APPROVE_OPTION) {
			File file = fc.getSelectedFile();
			try {
				model.loadFile(file.getPath());
			} catch (Exception ex) {
			}
		}
	}
	
	private void undo(){
		if(numSimpleChars > 0) {
			try {
												view.getStyledoc().remove(
													view.getStyledoc().getLength()-1,
													1);
											} catch (BadLocationException e) {
												
												e.printStackTrace();
											}
			model.handleBackspace();
			numSimpleChars--;
		}
		else
			model.undo();
	}

	public void actionPerformed(ActionEvent e) {
		// The strings compared against indicate the action performed.
		if (e.getActionCommand() == "Save") {
			runSaveDialog();

		} else if (e.getActionCommand() == "Load") {
			runLoadDialog();
		} else if (e.getActionCommand() == "Bold") {
			try {
				model.handleBold();
			} catch (Exception ex) {
			}
		} else if (e.getActionCommand() == "Italics") {
			try {
				model.handleItalic();
			} catch (Exception ex) {
			}
		} else if (e.getActionCommand() == "Underline") {
			try {
				model.handleUnderline();
			} catch (Exception ex) {
			}
		} else if (e.getActionCommand() == "Cut") {
			try {
				String cutText = model.cutSelection();
				StringSelection s = new StringSelection(cutText);
				Toolkit.getDefaultToolkit().getSystemClipboard().setContents(s,s);
			} catch (Exception ex) {
			}
		} else if (e.getActionCommand() == "Paste") {
			String clip = new String();
			try {
				clip = (String)(Toolkit.getDefaultToolkit().getSystemClipboard().getContents(new Object()).getTransferData(DataFlavor.stringFlavor));
			} catch (Exception e1) {
				
				e1.printStackTrace();
			} 
			for (int i = 0; i < clip.length(); i++) {
				model.appendChar(clip.charAt(i));
				
			}
			try {
				view.getStyledoc().insertString(view.textPane.getCaretPosition(), clip,
																				(view.getStyledoc()).getStyle(
																					model.getCurrentArtifact().getStyleString()));
			} catch (BadLocationException e2) {
				
				e2.printStackTrace();
			}
		} else if (e.getActionCommand() == "Bullets") {
			try {

				view.getStyledoc().insertString(
					view.getStyledoc().getLength(),
					model.handleBullet(),
					view.getStyledoc().getStyle(
						model.getCurrentArtifact().getStyleString()));
			} catch (Exception ex) {
			}
		}
		else if (e.getActionCommand() == "Header") {
			runHeaderDialog();
		}
		else if (e.getActionCommand() == "Undo") {
			undo();
		}
		view.textPane.requestFocus();

	}

	/* (non-Javadoc)
	 * @see java.awt.event.MouseListener#mouseClicked(java.awt.event.MouseEvent)
	 */
	public void mouseClicked(MouseEvent arg0) {
		
		int position = view.textPane.getCaretPosition();
		System.out.println(position);
					model.updatePosition(position);
	}

	/* (non-Javadoc)
	 * @see java.awt.event.MouseListener#mouseEntered(java.awt.event.MouseEvent)
	 */
	public void mouseEntered(MouseEvent arg0) {
	
		
	}

	/* (non-Javadoc)
	 * @see java.awt.event.MouseListener#mouseExited(java.awt.event.MouseEvent)
	 */
	public void mouseExited(MouseEvent arg0) {
		
		
	}

	/* (non-Javadoc)
	 * @see java.awt.event.MouseListener#mousePressed(java.awt.event.MouseEvent)
	 */
	public void mousePressed(MouseEvent arg0) {
		
		
	}

	/* (non-Javadoc)
	 * @see java.awt.event.MouseListener#mouseReleased(java.awt.event.MouseEvent)
	 */
	public void mouseReleased(MouseEvent arg0) {

	
	}
		
	
	
}
