/*
 * Created on Mar 7, 2004
 *
 * To change the template for this generated file go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
package test;

import java.io.FileInputStream;
import java.io.IOException;

import MVC.ApplicationMainView;
import MVC.Controller;

import junit.framework.TestCase;
import wikiviz.ArtifactList;

/**
 * @author Larry Benchimol
 *
 * To change the template for this generated type comment go to
 * Window&gt;Preferences&gt;Java&gt;Code Generation&gt;Code and Comments
 */
public class ApplicationLauncherTest extends TestCase {

	/**
	 * Constructor for ApplicationLauncherTest.
	 * @param arg0
	 */
	public ApplicationLauncherTest(String arg0) {
		super(arg0);
	}

	public void testSaveandLoad() { //System test 1

		//First we create a model
		ArtifactList model = new ArtifactList();
		model.addObserver(new ApplicationMainView());
		model.addObserver(new Controller());

		//We simulate typing the letters p,l,a,i,n, ,t,e,x,t
		appendHelper(model, "plain text");
		
		//Simulate typing enter.
		//We use handleEnter because we cannot manipulate what the controller reads from the keyboard
		//without actually typing it, so in an automatic test, we must use handleEnter
		model.handleEnter();
		
		//HandleBullet simulates hitting the bullet button or pressing CTRL+l
		model.handleBullet();
		model.handleBullet();
		
		//All of the text represents what you should see, so at this point, we are at a level 2 bullet
		//By level 2 bullet, we mean that this is a bullet that is indented
		//Level 3 means indented twice, this is a 1-based counting system
		appendHelper(model, "this is a level 2 bullet");
		model.handleEnter();
		model.handleBullet();
		
		//Here we show that there can be nested formats (bold and underline at same time)
		//inside of a bullet.  In other words, we can have a bullet that has multiple formatted text
		model.handleBold();
		model.handleUnderline();
		appendHelper(model, "this is a level 3 bu bullet");
		
		//We can also use shift+tab to unindent our bullets and it does not interrupt our formatting
		model.handleEnter();
		model.handleEndBullet();
		model.handleEndBullet();
		model.handleUnderline();
		appendHelper(model, "this is bold level 1 bullet");
		model.handleEnter();
		model.handleEndBullet();
		model.handleItalic();
		appendHelper(model, "this is non-bullet bold italic");
		//We compare this to the desired output
		assertEquals(
			"plain text\n**this is a level 2 bullet\n***<b><u>this is a level 3 bu bullet</u></b>\n*<b>this is bold level 1 bullet</b>\n<b><i>this is non-bullet bold italic</i></b>",
			model.toWiki());

		
	}

	public void testSave() throws IOException {//System test 2
		//This test case is designed to test the character by character output of the save functionality, 
		//in addition to many formatting texts 
		ArtifactList model = new ArtifactList();
		model.addObserver(new ApplicationMainView());
		model.addObserver(new Controller());

		appendHelper(model, "plain text");
		model.handleEnter();
		model.handleBullet();
		model.handleBullet();
		appendHelper(model, "this is a level 2 bullet");
		model.handleEnter();
		model.handleBullet();
		model.handleBold();
		model.handleUnderline();
		appendHelper(model, "this is a level 3 bu bullet");
		model.handleEnter();
		model.handleEndBullet();
		model.handleEndBullet();
		model.handleUnderline();
		appendHelper(model, "this is bold level 1 bullet");
		model.handleEnter();
		model.handleEndBullet();
		model.handleItalic();
		appendHelper(model, "this is non-bullet bold italic");
		try {
			model.outputWikiDoc("c:\\testing.wiki");
		} catch (Exception e) {
			
			e.printStackTrace();
		}

		FileInputStream fin = new FileInputStream("c:\\testing.wiki");

		//After saving the file, we open it for reading
		String wikiSyntax = model.toWiki();

		//We compare it character by character to the desired output
		for (int i = 0; i < wikiSyntax.length(); i++) {
			assertEquals(wikiSyntax.charAt(i), fin.read());
		}

	}

	private void appendHelper(ArtifactList al, String s) {
		int i = 0;
		while (i < s.length())
			al.appendChar(s.charAt(i++));
	}

}
